#!/usr/bin/env ruby

require 'test/unit'
require 'flexmock'

class TestFlexMock < Test::Unit::TestCase
  def setup
    @mock = FlexMock.new
  end

  def test_handle
    args = nil
    @mock.mock_handle(:hi) { |a, b| args = [a,b] }
    @mock.hi(1,2)
    assert_equal [1,2], args
  end

  def test_handle_no_block
    @mock.mock_handle(:blip)
    @mock.blip
    assert true, "just checking for failures"
  end

  def test_called_with_block
    called = false
    @mock.mock_handle(:blip) { |block| block.call }
    @mock.blip { called = true }
    assert called, "Block to blip should be called"
  end

  def test_return_value
    @mock.mock_handle(:blip) { 10 }
    assert_equal 10, @mock.blip
  end

  def test_handle_missing_method
    expected_error = (VERSION >= "1.8.0") ? NoMethodError : NameError
    ex = assert_raises(expected_error) {
      @mock.not_defined
    }
    assert_match /not_defined/, ex.message
  end

  def test_ignore_missing_method
    @mock.mock_ignore_missing
    @mock.blip
    assert true, "just checking for failures"
  end

  def test_method_count
    @mock.mock_handle(:blip)
    @mock.mock_handle(:bing)
    @mock.blip
    @mock.blip
    @mock.bing
    assert_equal 1, @mock.mock_count(:bing)
    assert_equal 2, @mock.mock_count(:blip)
  end

  def test_good_counts
    @mock.mock_handle(:blip, 3)
    @mock.blip
    @mock.blip
    @mock.blip
    @mock.mock_verify
  end

  def test_bad_counts
    @mock.mock_handle(:blip, 3)
    @mock.blip
    @mock.blip
    ex = nil
    begin
      @mock.mock_verify
    rescue Test::Unit::AssertionFailedError => err
    end
    assert_not_nil err
  end

  def test_zero_counts
    assert_raises(Test::Unit::AssertionFailedError) do
      FlexMock.use { |m|
	m.mock_handle(:blip, 0)
	m.blip
      }
    end
  end

  def test_use
    assert_raises(Test::Unit::AssertionFailedError) {
      FlexMock.use do |m|
	m.mock_handle(:blip, 2)
	m.blip
      end
    }
  end

  def test_sequential_values
    values = [1,4,9,16]
    @mock.mock_handle(:get) { values.shift }
    assert_equal 1, @mock.get
    assert_equal 4, @mock.get
    assert_equal 9, @mock.get
    assert_equal 16, @mock.get
  end

end
